#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from gettext import gettext as _
import os
import sys
import argparse

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Gio

from .configuration import *
from .windows import ApplicationWindow
from .windows import PrefernecesWindow
from .windows import ShortcutsWindow
from .windows import AboutWindow

class Application(Gtk.Application):

    def __init__(self):
        Gtk.Application.__init__(self,
                                 application_id = APP_NAMESPACE,
                                 flags = Gio.ApplicationFlags.HANDLES_COMMAND_LINE)

        #Gio.Application.get_default().set_value("test", "example")

        #print (Gio.Application.get_default().set_value("test"))

        self.window = None
        self.args = None
        self.settings = None

        #if self.settings is None:
        #    self.settings = Gio.Settings(APP_NAMESPACE)
        #    Gio.Settings.reg
        # Gio.Settings("org.app")
        # Gio.Settings.new("org.app"_
        # Gio.Application
        # Gio.SettingsSchemaSource
        # configparser (https://docs.python.org/2/library/configparser.html)

    """
    Gtk.Application startup handler
    """
    def do_startup(self):
        Gtk.Application.do_startup(self)

        self.build_menu()

    """
    Gtk.Application activate handler
    The starts the window if one does not exist and shows them all.
    """
    def do_activate(self):
        # Do command line
        if self.args.version:
            print(APP_VERSION)
            return

        if self.args.debug:
            print("Starting debug...")

        # Window already exists, return that
        if self.window:
            return

        # Else create a new application window.
        self.window = ApplicationWindow(self)

        # Connect quit event (normally already done but lets do it anyway)
        self.window.connect("delete_event", self.on_quit)

        # Build UI
        self.window.build()

        # Show all the windows.
        # You MUST use .show_all() for Gtk.headerBar() to work!
        # .present() is a wrapper of .show() and .show() only shows the current window.
        self.window.show_all()

    """
    Gtk.Application command line handler
    Called if Gio.ApplicationFlags.HANDLES_COMMAND_LINE is set.
    """
    def do_command_line(self, args):
        Gtk.Application.do_command_line(self, args)

        # Start parsing
        parser = argparse.ArgumentParser(description=APP_DESCRIPTION,
                                         prog=APP_COMMAND)

        # Add debug option (-d/--debug)
        parser.add_argument('-d', '--debug',
                            action='store_true',
                            default=False,
                            dest="debug",
                            help=_("run in debug mode and show extra output")
                           )

        # Add version option (-v/--version)
        parser.add_argument('-v', '--version',
                            action='store_true',
                            default=False,
                            dest="version",
                            help=_("return the current version")
                           )

        # Store arguments so application can use them
        # We strip off the first argument (the file name)
        self.args = parser.parse_args(args.get_arguments()[1:])

        # Since we are calling do_command_line() we need to call do_activate()
        # For the program to continue
        self.do_activate()

        return 0

    """
    Build Gio Menu and add acclerator actions.
    """
    def build_menu(self):
        # Init Gio menu (a.k.a app-menu)
        menu = Gio.Menu()

        # Menu Sections & Items
        # Each sub array is a section
        menu_items = [
            [
                {
                    "action": "preferences",
                    "label": _("Preferences"),
                    "callback": self.on_preferences,
                    "accelerators": ["<Control>p", "<Control>comma"],
                    "description": _("Open preferences window.")
                }
            ], [
                {
                    "action": "shortcuts",
                    "label": _("Keyboard Shortcuts"),
                    "callback": self.on_shortcuts,
                    "accelerators": ["<Control>F1", "<Control><Shift>question"],
                    "description": _("Open shortcuts window."),
                    "show": (Gtk.get_major_version() >= 3 and Gtk.get_minor_version() >= 20)
                },
                {
                    "action": "help",
                    "label": _("Help"),
                    "callback": self.on_help,
                    "accelerators": ["F1"],
                    "description": _("Open help window.")
                },
                {
                    "action": "about",
                    "label": _("About"),
                    "callback": self.on_about,
                    "accelerators": ["F2"],
                    "description": _("Open about window.")
                },
                {
                    "action": "quit",
                    "label": _("Quit"),
                    "callback": self.on_quit,
                    "accelerators": ["<Control>q"],
                    "description": _("Quit application.")
                },
            ]
        ]

        menu_sections = []
        menu_contents = []

        # Create a section for the contained actions
        for section, actions in enumerate(menu_items):

            menu_contents.append(section)
            menu_contents[section] = Gio.Menu()

            # Build callbacks and acclerators
            for action in actions:
                # Special show key to show/hide (enable/disable) the action
                # Default is True
                if "show" in action:
                    if action["show"] == False:
                        continue

                # Create menu item
                menu_contents[section].append(action["label"], "app.{}".format(action["action"]))

                # Add action to menu item
                simpleAction = Gio.SimpleAction.new(action["action"], None)
                simpleAction.connect('activate', action["callback"])
                self.add_action(simpleAction)

                # Add accelerator action
                if "accelerators" in action:
                    self.set_accels_for_action("app.{}".format(action["action"]), action["accelerators"])

                # Add acclerator to shortcut window
                if "description" in action:
                    ShortcutsWindow.add_action("app.{}".format(action["action"]), action["description"])

            # Add sub-menu
            menu_sections.append(section)
            menu_sections[section] = Gio.MenuItem.new_section(None, menu_contents[section])
            menu.append_item(menu_sections[section])

        # Add app-menu to app
        self.set_app_menu(menu)


    """
    Callbacks for Gio.Menu
    """
    def on_preferences(self, action = None, param = None):
        preferences = PrefernecesWindow(main_window=self.window)
        preferences.build()
        preferences.show_all()

    def on_help(self, action = None, param = None):
        Gtk.show_uri(None, APP_MANUAL_URL_OFFLINE, Gdk.CURRENT_TIME)

    def on_shortcuts(self, action = None, param = None):
        shortcuts = ShortcutsWindow(main_window=self.window, app=self)
        shortcuts.build()
        shortcuts.show_all()

    def on_about(self, action = None, param = None):
        about = AboutWindow(main_window=self.window)
        about.build()
        about.show_all()

    """
    Gtk.Application shutdown handler
    Do clean up before the application is closed, useful when you need
    to close a database connection or save work for the application.
    This is triggered when self.quit() is called.
    """
    def do_shutdown(self):
        Gtk.Application.do_shutdown(self)

    """
    Gtk.Application quit handler
    Quit the application when requested.
    """
    def on_quit(self, action = None, param = None):
        self.quit()

def main():
    app = Application()
    exit_status = app.run(sys.argv)
    sys.exit(exit_status)
