#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import configparser
from gettext import gettext as _

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GLib

#from .configuration import *
#from .utils import *

class PreferencesException(Exception):
    def __init__(self, reason):
        self.reason = reason

class PreferencesWindowException(Exception):
    def __init__(self, reason):
        self.reason = reason

class PreferencesWindow(Gtk.Window):

    _show_tabs = True
    _show_search = True
    pages = {}
    flowbox = {}
    notebook = None

    def __init__(self, main_window = None):
        Gtk.Window.__init__(self)

        self.main_window = main_window
        self.connect("delete-event", self._quit)
        self.connect("key-press-event", self._keypress)

        """
        Build Settings
        """
        self.set_title(_("Preferences"))
        self.set_default_size(480, 400)
        self.set_modal(True)
        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.set_transient_for(self.main_window)
        self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)

        """
        Build HeaderBar
        """
        self.headerbar = Gtk.HeaderBar()
        self.headerbar.set_title(_("Preferences"))
        self.headerbar.set_show_close_button(True)
        self.set_titlebar(self.headerbar)

        """
        Build Content
        """
        self.build()

    def _keypress(self, widget, event):
        mod = Gtk.accelerator_get_label(event.keyval, event.state)

        if mod == "Escape" or mod == "Mod2+Escape":
            #self.destroy()
            Gtk.main_quit()

        if self._show_search == True:
            if mod == "Ctrl+F" or mod == "Ctrl+Mod2+F":
                self._on_search_toggle()

    def _quit(self, widget, event):
        #self.destroy()
        Gtk.main_quit()

    def present(self):
        self.show_all()

    def show(self):
        self.show_all()

    def show_all(self):
        """
        Build Search
        We load this after all the preferences have been added to the window
        """
        if self._show_search == True:
            self._build_search()

        """
        No show window
        """
        Gtk.Window.show_all(self)

    def build(self):
        """
        Build Layout
        """
        self.layout = Gtk.Grid()
        self.layout.set_orientation(Gtk.Orientation.VERTICAL)
        self.layout.set_border_width(0)
        self.layout.set_column_homogeneous(True)
        self.add(self.layout)

    def set_show_tabs(self, show):
        if isinstance(show, (int, float)) == False:
            raise PreferencesWindowException("set_show_tabs() only accepts booleans.")

        self._show_tabs = show

    def set_show_search(self, show):
        if isinstance(show, (int, float)) == False:
            raise PreferencesWindowException("set_show_search() only accepts booleans.")

        self._show_search = show

    def set_row(self, widget, page = None):
        # Create or find container
        container = self.set_page(page)
        # Build row (FlowBoxChild)
        row = Gtk.FlowBoxChild()
        row.set_can_focus(False)
        row.set_margin_left(10)
        row.set_margin_right(10)
        row.set_margin_top(5)
        row.set_margin_bottom(5)
        row.add(widget)
        # Add widget to row
        container.add(row)

    def set_page(self, page = None):
        # Always defaul to General tab/page
        if page == None:
            page = "General"

        # Create notebook if it does not exists already
        if self.notebook == None:
            # Create notebook
            self.notebook = Gtk.Notebook()
            self.notebook.set_tab_pos(Gtk.PositionType.TOP) # LEFT, RIGHT, TOP, BOTTOM
            self.notebook.set_scrollable(True)
            self.notebook.set_show_border(False)
            self.notebook.set_show_tabs(self._show_tabs)

            # Add notebook to layout
            self.layout.attach(self.notebook, 0, 1, 1, 1)

        # Create tab & page if it does not exists
        if page not in self.pages:
            # Add containers for each page
            self.pages[page] = Gtk.ScrolledWindow()
            self.pages[page].set_vexpand(True)
            self.pages[page].set_border_width(0)

            # Add flowbox for each page
            self.flowbox[page] = Gtk.FlowBox()
            self.flowbox[page].set_valign(Gtk.Align.START)
            self.flowbox[page].set_max_children_per_line(1)
            self.flowbox[page].set_selection_mode(Gtk.SelectionMode.NONE)
            self.flowbox[page].set_margin_top(5)

            # Add flowbox to container
            self.pages[page].add(self.flowbox[page])

            # Add container to notebook
            self.notebook.append_page(self.pages[page], Gtk.Label(_(page)))

        if len(self.flowbox.keys()) > 1:
            self.notebook.set_show_tabs(True)

        # Add row to flowbox
        return self.flowbox[page]

    def set(self,
            title = None,
            type = None,
            callback = None,
            options = None,
            page = None,
            value = None,
            description = None,
            section = None,
            keywords = None):

        # Create list row content box
        box = Gtk.Box()
        box.set_orientation(Gtk.Orientation.HORIZONTAL)
        box.set_homogeneous(True)

        # Create list row content box label
        label = Gtk.Label()
        label.set_text(_(title))
        label.set_halign(Gtk.Align.START)
        label.set_valign(Gtk.Align.CENTER)
        box.pack_start(label, True, True, 0)

        # Create list row content box switch
        switch = Gtk.Switch()
        switch.set_halign(Gtk.Align.END)
        switch.set_valign(Gtk.Align.CENTER)
        switch.set_active(True)
        switch.connect("notify::active", callback)
        box.set_focus_child(switch)
        box.pack_start(switch, False, True, 0)

        self.set_row(box, page)

    def _build_search(self):
        self.search_icon = Gtk.Image()
        self.search_icon.set_from_icon_name("edit-find-symbolic", Gtk.IconSize.BUTTON)
        self.search_button = Gtk.ToggleButton()
        self.search_button.add(self.search_icon)
        self.search_button.connect("toggled", self._on_search_toggle)
        self.headerbar.pack_start(self.search_button)

        self.searchbar = Gtk.SearchBar()
        self.searchbar.set_hexpand(True)
        self.searchbar.set_halign(Gtk.Align.FILL)
        self.layout.attach(self.searchbar, 0, 0, 1, 1)

        self.search_entry = Gtk.SearchEntry()
        self.search_entry.set_placeholder_text(_("Search Preferences"))
        self.search_entry.connect("search-changed", self._on_search)

        self.searchbar.connect_entry(self.search_entry)
        self.searchbar.add(self.search_entry)

    def _on_search_toggle(self, button = None):
        if self.searchbar.get_search_mode():
            self.searchbar.set_search_mode(False)
            self.search_entry.set_text("")
        else:
            self.searchbar.set_search_mode(True)
            self.search_entry.grab_focus()

    def _on_search(self, entry):
        search = entry.get_text().lower()

        for page in self.flowbox:
            self.flowbox[page].set_filter_func(self._on_search_filter, search)

    def _on_search_filter(self, lista, search_data):
        if not search_data:
            return lista

        # TODO: Validate
        items = lista.get_child().get_children()[0].get_text().lower()
        if search_data in items:
            return lista

class Preferences(object):

    def __init__(self, configFile = None):

        try:
            self.config = configparser.ConfigParser()
        except:
            raise PreferencesException("Unable to load configparser, make sure you have configparser installed.")

        if configFile != None:
            self.file(configFile)

    def file(self, configFile = None):
        # Get path
        ## No path use default (current working dir)
        # Check if exists
        ## If not create, or don't

        print(configFile)

    def set_section(self, section):
        self.config.add_section(section)

    def set(self, section, key, value):
        if self.config.has_section(section) == False:
            self.set_section(section)

        self.config.set(section, key, value)

    def get(self, section, key):
        return self.config.get(section, key)

    def items(self, section):
        if self.config.has_section(section) == False:
            return None

        return self.config.items(section)



def on_switch_activated(switch, gparam):
    if switch.get_active():
        state = "on"
    else:
        state = "off"

    print("Switch was turned", state)


preferences = Preferences()
preferences.file("~/.config/gas/importer/config.conf")

#preferences = configparser.ConfigParser()
#preferences.add_section("General")

preferences.set("General", "example", "foo")
preferences.set("General", "example2", "bar")
print(preferences.items("General"))
print(preferences.get("General", "example"))

for key in preferences.config["General"]:
    print(preferences.get("General", key))

#print(preferences["General"])

#for key in preferences["General"]: print(preferences.get("General", key))



#preferences = configparser.ConfigParser()
#preferences['DEFAULT'] = {'ServerAliveInterval': '45',
#                          'Compression': 'yes',
#                          'CompressionLevel': '9'}



#preferences.file()
#preferences.section()
#preferences.set("General", "example", "foo")
#preferences["General"] = {"example2": "bar"}
#print(preferences.get("example"))
#print(preferences["General"]["example2"])
#print(preferences.sections())



preferencesWindow = PreferencesWindow()
preferencesWindow.set_show_tabs(True)
preferencesWindow.set_show_search(True)

widget1 = Gtk.Label()
widget1.set_text("Test")
widget2 = Gtk.Label()
widget2.set_text("Test 2")
widget3 = Gtk.Label()
widget3.set_text("Test 3")
widget4 = Gtk.Label()
widget4.set_text("Test 4")

"""
preferencesWindow.set_row(widget1)
preferencesWindow.set_row(widget2)
preferencesWindow.set_row(widget3)
preferencesWindow.set_row(widget4)
"""
preferencesWindow.set(
    title = "Title of option",
    description = "This is the description.",
    type = "switch",
    callback = on_switch_activated,
    page = "General"
)
preferencesWindow.set(
    title = "Title of option 2",
    description = "This is the description.",
    type = "switch",
    callback = on_switch_activated,
    page = "General"
)
preferencesWindow.set(
    title = "Title of option 3",
    description = "This is the description.",
    type = "switch",
    callback = on_switch_activated,
    page = "Video"
)
preferencesWindow.set(
    title = "Title of option 4",
    description = "This is the description.",
    type = "switch",
    callback = on_switch_activated
)

"""
preferencesWindow.set_row(widget1, page = "General")
preferencesWindow.set_row(widget2, page = "General")
preferencesWindow.set_row(widget3, page = "Video")
preferencesWindow.set_row(widget4)
"""
preferencesWindow.show_all()
Gtk.main()








"""
preferencesWindow = PreferencesWindow(main_window=self.window)

preferencesWindow.set(
    section = "",
    label = "",
    description = "",
    type = "",
    value = "",
    default = "",
    callback = "",
    options = "",
    tab = ""
)

widget = Gtk.Label()
widget.set_text("Test")

preferencesWindow.set_row(widget, tab = "General")


preferencesWindow.show_all()
Gtk.main()
"""




"""
window = PreferencesWindow()

window.add_preference("Title of option", "switch", on_switch_activated, None)
window.add_preference("Title of option 2", "switch", on_switch_activated, None)
window.add_preference("Title of option 3", "switch", on_switch_activated, None)
window.add_preference("Title of option 4", "switch", on_switch_activated, None)
window.add_preference("Title of option 5", "switch", on_switch_activated, None)
window.add_preference("Title of option 6", "switch", on_switch_activated, None)

window.add_preference("Title of option", "switch", on_switch_activated, None, "General")
window.add_preference("Title of option 2", "switch", on_switch_activated, None, "General")
window.add_preference("Title of option 3", "switch", on_switch_activated, None, "General")


window.show_all()
Gtk.main()
"""


"""
if is_linux():
    #config_file = os.path.expanduser("~/.config/{0}/{0}.ini".format(APP_COMMAND))
    config_file = "config.ini"

preferences = Preferences()
preferences.file(config_file)

preferences.set(section, key, value, default, {
    tab: "",
    label: "",
    type: "",
    callback: "",
    description: "",
    options: ""
})

pref = preferences.get()
preferences.pref[name].value
print (pref.value)
print (pref.default)

preferences.window(main_window=self.window)


preferencesWindow = PreferencesWindow(main_window=self.window)

preferencesWindow.set(
    section = "",
    label = "",
    description = "",
    type = "",
    value = "",
    default = "",
    callback = "",
    options = "",
    tab = ""
)

preferencesWindow.show_all()















# Inside preferences.window()
self.window = PreferencesWindow(main)

for pref in self.preferencesWindow:
    self.window.set(label, type, callback, options, section, description)
"""
